import cv2 as cv
import numpy as np
import os
from pathlib import Path
from PySide2 import QtWidgets, QtGui, QtCore
from PySide2.QtCore import QSize

from galleries.galleries_management import GalleriesManagement
from galleries.gallery import Gallery
from galleries.annotations_parsers.file_name_parser import FileNameSepParser
from galleries.igallery import IGallery
from galleries.images_providers.local_files_image_providers import LocalFilesImageProvider
from mnd_qtutils.qtutils import setup_widget_from_ui, icon_from_image
import mnd_utils.image
from galleries_qt.gallery_wizard import GalleryWizard


class GalleriesManager(QtWidgets.QWidget):

    def __init__(self, parent=None):
        super(GalleriesManager, self).__init__(parent=parent)

        ui_file_path = os.path.join(Path(__file__).parent, 'galleries_manager.ui')
        self._widget: QtWidgets.QWidget = setup_widget_from_ui(ui_file_path, self)

        self._gallery_dir_edit: QtWidgets.QLineEdit = self._widget.gal_dir_edit
        self._gallery_dir_edit.setText(GalleriesManagement.get_galleries_folder())

        self._gallery_dir_button: QtWidgets.QPushButton = self._widget.gal_dir_button
        self._gallery_dir_button.setIcon(self.style().standardIcon(QtWidgets.QStyle.SP_DialogOpenButton))
        self._gallery_dir_button.clicked.connect(self._change_galleries_folder)

        self._galleries_list: QtWidgets.QListWidget = self._widget.galleries_list
        self._galleries_list.itemSelectionChanged.connect(self._on_gallery_selection_changed)

        self._gallery_wizard: GalleryWizard = GalleryWizard()
        self._gallery_wizard.setEnabled(False)
        self._gallery_wizard.setSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        self._gallery_wizard.setMinimumHeight(self._gallery_wizard.minimumSizeHint().height())
        self._wizard_container: QtWidgets.QWidget = self._widget.wizard_container
        self._wizard_container.layout().addWidget(self._gallery_wizard)

        self._new_button: QtWidgets.QPushButton = self._widget.new_button
        self._new_button.clicked.connect(self._add_empty_gallery)

        self._save_button: QtWidgets.QPushButton = self._widget.save_button
        self._save_button.clicked.connect(self._save_gallery)
        self._save_button.setEnabled(False)

        self._remove_button: QtWidgets.QPushButton = self._widget.remove_button
        self._remove_button.clicked.connect(self._remove_gallery)
        self._remove_button.setEnabled(False)

        self._update_galleries_list()

    @QtCore.Slot()
    def _change_galleries_folder(self):
        workdir = os.getcwd()
        galleries_folder = QtWidgets.QFileDialog.getExistingDirectory(
            self,
            self.tr("Seleccionar ubicación de las galerías"),
            workdir,
        )

        if os.path.exists(galleries_folder):
            GalleriesManagement.set_galleries_folder(galleries_folder)
            self._gallery_dir_edit.setText(galleries_folder)
            self._update_galleries_list()

    @QtCore.Slot()
    def _on_gallery_selection_changed(self):
        any_gallery_selected = self._is_any_gallery_selected()
        if any_gallery_selected:
            gallery_item: GalleryListWidgetItem = self._get_selected_gallery()
            gallery_name = gallery_item.gallery_name
            gallery = gallery_item.gallery
            self._gallery_wizard.set_gallery(gallery_name, gallery)
        else:
            self._gallery_wizard.clear()

        self._gallery_wizard.setEnabled(any_gallery_selected)
        self._save_button.setEnabled(any_gallery_selected)
        self._remove_button.setEnabled(any_gallery_selected)

    def _is_any_gallery_selected(self):
        selected_items = self._galleries_list.selectedItems()
        return len(selected_items) > 0

    def _get_selected_gallery(self):
        selected_items = self._galleries_list.selectedItems()
        return selected_items[0]

    def _update_galleries_list(self):
        self._clear_galleries_list()
        galleries = GalleriesManagement.load_galleries()
        for gallery_name, gallery in galleries:
            self._add_gallery_to_list(gallery_name, gallery)

    def _add_gallery_to_list(self, gallery_name: str, gallery: IGallery):
        icon_size = self._galleries_list.iconSize()
        gallery_list_item = GalleryListWidgetItem(gallery_name, gallery, icon_size)
        self._galleries_list.addItem(gallery_list_item)

    @QtCore.Slot()
    def _add_empty_gallery(self):
        images_provider = LocalFilesImageProvider('')
        parser = FileNameSepParser([], '')
        gallery = Gallery("", images_provider, parser)
        self._add_gallery_to_list('', gallery)

    @QtCore.Slot()
    def _save_gallery(self):
        any_gallery_selected = self._is_any_gallery_selected()
        if any_gallery_selected:
            gallery_item: GalleryListWidgetItem = self._get_selected_gallery()
            gallery_name = self._gallery_wizard.get_name()
            gallery = self._gallery_wizard.get_gallery()
            gallery_item.gallery_name = gallery_name
            gallery_item.gallery = gallery
            GalleriesManagement.save_gallery(gallery_name, gallery)

    @QtCore.Slot()
    def _remove_gallery(self):
        any_gallery_selected = self._is_any_gallery_selected()
        if any_gallery_selected:
            gallery_item: GalleryListWidgetItem = self._get_selected_gallery()
            gallery_name = gallery_item.gallery_name
            GalleriesManagement.remove_gallery_by_name(gallery_name)
            self._galleries_list.takeItem(self._galleries_list.row(gallery_item))

    def _clear_galleries_list(self):
        self._galleries_list.clear()


class GalleryListWidgetItem(QtWidgets.QListWidgetItem):

    def __init__(self, gallery_name: str, gallery: IGallery, icon_size: QSize):
        icon = self._get_first_image_icon(gallery, icon_size)
        super().__init__(icon, gallery_name)
        self._gallery_name = gallery_name
        self._gallery = gallery

    @property
    def gallery_name(self):
        return self._gallery_name

    @gallery_name.setter
    def gallery_name(self, value: str):
        self._gallery_name = value
        self.setText(value)

    @property
    def gallery(self):
        return self._gallery

    @gallery.setter
    def gallery(self, value):
        self._gallery = value
        icon = self._get_first_image_icon(value, self.sizeHint())
        self.setIcon(icon)

    def _get_first_image_icon(self, gallery: IGallery, icon_size: QSize) -> QtGui.QIcon:
        indices = gallery.get_indices()
        try:
            first_image_index = next(indices)
            image = cv.imread(first_image_index)
        except:
            image = np.ones((64, 64, 3)).astype(np.uint8) * 127
        image = cv.cvtColor(image, cv.COLOR_BGR2RGB)
        image = mnd_utils.image.fit_in_size(image, (icon_size.width(), icon_size.height()))
        icon = icon_from_image(image)
        return icon


if __name__ == '__main__':
    import sys
    from PySide2.QtWidgets import QApplication
    from PySide2.QtWidgets import QWidget, QVBoxLayout

    app = QApplication(sys.argv)

    window = QWidget()
    window.setMinimumSize(100, 100)
    layout = QVBoxLayout()
    window.setLayout(layout)

    panel = GalleriesManager()
    layout.addWidget(panel)

    window.show()

    sys.exit(app.exec_())



from galleries.galleries_management import GalleriesManagement
from PySide2 import QtWidgets, QtCore


class GalleriesCombobox(QtWidgets.QWidget):
    event_gallery_selection_changed = QtCore.Signal()

    def __init__(self, parent=None):
        super(GalleriesCombobox, self).__init__(parent=parent)

        self._combobox = QtWidgets.QComboBox()
        self._combobox.currentIndexChanged.connect(self._on_selection_changed)

        layout = QtWidgets.QHBoxLayout()
        layout.addWidget(self._combobox)
        self.setLayout(layout)

        self._populate_galleries()

    def _populate_galleries(self):
        galleries = GalleriesManagement.load_galleries()
        for gallery_name, gallery in galleries:
            self._combobox.addItem(gallery_name, gallery)

    @QtCore.Slot()
    def _on_selection_changed(self, index: int):
        self.event_gallery_selection_changed.emit()

    def current_gallery_selected(self):
        gallery = self._combobox.currentData()
        gallery_name = self._combobox.currentText()
        return gallery_name, gallery



from galleries.annotations_parsers.file_name_parser import FileNameSepParser
from propsettings_qt.setting_widget_retrieval import register_object_drawer

from galleries_qt.parser_widgets.file_name_parser_widget import FileNameParserWidget


register_object_drawer(FileNameSepParser, FileNameParserWidget)



from galleries.igallery import IGallery
from mnd_qtutils.qtutils import setup_widget_from_ui
import os
from pathlib import Path
from pyrulo_qt.ui_configurable_selector import ConfigurableSelector
from PySide2 import QtWidgets, QtGui, QtCore

import galleries_qt


class GalleryWizard(QtWidgets.QWidget):

    def __init__(self, parent=None):
        super(GalleryWizard, self).__init__(parent=parent)

        self._dirty = False

        ui_file_path = os.path.join(Path(__file__).parent, 'gallery_wizard.ui')
        self._widget: QtWidgets.QWidget = setup_widget_from_ui(ui_file_path, self)

        self._name_edit: QtWidgets.QLineEdit = self._widget.name_edit
        self._name_edit.setValidator(QtGui.QRegExpValidator('[A-Za-z0-9_áéíóúÁÉÍÓÚ]*'))
        self._name_edit.textEdited.connect(self._set_dirty)

        self._gallery_container: QtWidgets.QWidget = self._widget.gallery_container
        self._gallery_selector = ConfigurableSelector(base_class=IGallery)
        self._gallery_selector.eventObjectSelected.connect(self._on_gallery_changed)
        self._gallery_container.layout().addWidget(self._gallery_selector)

    def is_dirty(self):
        dirty = self._dirty
        return dirty

    def set_gallery(self, gallery_name: str, gallery: IGallery):
        self._name_edit.setText(gallery_name)
        self._set_gallery_ui_by_gallery(gallery)
        self._dirty = False

    def get_gallery(self) -> IGallery:
        gallery_name = self._name_edit.text()
        gallery: IGallery = self._gallery_selector.current_object()
        gallery.set_name(gallery_name)
        return gallery

    def get_name(self) -> str:
        return self._name_edit.text()

    def clear(self):
        self._provider_selector.set_current_index(0)
        self._parser_selector.set_current_index(0)
        self._dirty = False

    def _set_gallery_ui_by_gallery(self, gallery: IGallery):
        gallery_class = type(gallery)
        self._gallery_selector.add_class(gallery_class)
        self._gallery_selector.set_object_for_class(gallery_class, gallery)
        self._gallery_selector.select_class(gallery_class)

    @QtCore.Slot()
    def _on_gallery_changed(self, index):
        self._set_dirty()

    def _set_dirty(self):
        self._dirty = True


if __name__ == '__main__':
    import sys
    from PySide2.QtWidgets import QApplication
    from PySide2.QtWidgets import QWidget, QVBoxLayout

    app = QApplication(sys.argv)

    window = QWidget()
    window.setMinimumSize(600, 500)
    layout = QVBoxLayout()
    window.setLayout(layout)

    panel = GalleryWizard()
    layout.addWidget(panel)

    window.show()

    sys.exit(app.exec_())



from galleries.annotations_parsers.gallery_annots_parsers import GalleryAnnotationsParser
from galleries.gallery import Gallery
from galleries.images_providers.gallery_images_provider import GalleryImagesProvider
from propsettings.configurable import register_as_setting
from pyrulo_qt import ClassSelector

import galleries_qt.parser_widgets


register_as_setting(Gallery, "_images_provider", setting_type=ClassSelector(GalleryImagesProvider), sort_order=0)
register_as_setting(Gallery, "_annots_parser", setting_type=ClassSelector(GalleryAnnotationsParser), sort_order=1)



import abc
from PySide2 import QtWidgets

from galleries.annotations_parsers.gallery_annots_parsers import GalleryAnnotationsParser


class GalleryAnnotationsParserView(QtWidgets.QWidget):

    @abc.abstractmethod
    def get_parser(self) -> GalleryAnnotationsParser:
        pass

    @abc.abstractmethod
    def populate_with_parser(self, parser: GalleryAnnotationsParser):
        pass

    @abc.abstractmethod
    def clear(self):
        pass

    @abc.abstractmethod
    def is_dirty(self) -> bool:
        pass



from PySide2 import QtWidgets
from galleries.annotations_parsers.gallery_annots_parsers import GalleryAnnotationsParser
from propsettings_qt.ui_settings_area import SettingsAreaWidget

from galleries_qt.gallery_annotations_parser_view import GalleryAnnotationsParserView


class SettingsParserWidget(GalleryAnnotationsParserView):

    def __init__(self, parent=None):
        super(SettingsParserWidget, self).__init__(parent=parent)
        self._settings_area = SettingsAreaWidget()

        self._layout = QtWidgets.QVBoxLayout()
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.addWidget(self._settings_area)

        self._parser = None

    def get_parser(self) -> GalleryAnnotationsParser:
        return self._parser

    def populate_with_parser(self, parser: GalleryAnnotationsParser):
        self._parser = parser
        self._settings_area.populate_object(self._parser)

    def clear(self):
        pass

    def is_dirty(self) -> bool:
        return False



import os
from pathlib import Path
from PySide2 import QtWidgets, QtCore

from galleries.annotations_parsers.file_name_parser import FileNameSepParser, GalleryAnnotationsParser
from mnd_qtutils.qtutils import setup_widget_from_ui
from propsettings_qt.object_drawers.object_drawer import ObjectDrawer

from galleries_qt.gallery_annotations_parser_view import GalleryAnnotationsParserView


class FileNameParserWidget(GalleryAnnotationsParserView, ObjectDrawer):

    SEP = ';'

    def __init__(self, parent=None):
        super(FileNameParserWidget, self).__init__(parent=parent)

        self._parser: FileNameSepParser = None
        self._dirty: bool = False

        ui_file_path = os.path.join(Path(__file__).parent, 'file_name_parser_widget.ui')
        self._widget: QtWidgets.QWidget = setup_widget_from_ui(ui_file_path, self)

        self._sep_edit: QtWidgets.QLineEdit = self._widget.sep_edit
        self._sep_edit.textEdited.connect(self._on_sep_changed)

        self._annot_edit: QtWidgets.QTextEdit = self._widget.annot_edit
        self._annot_edit.textChanged.connect(self._on_annots_changed)

    def get_parser(self) -> GalleryAnnotationsParser:
        sep = self._sep_edit.text()
        annots_names = self._get_annots()
        parser = FileNameSepParser(annots_names, sep)
        return parser

    def populate_with_parser(self, parser: FileNameSepParser):
        self._sep_edit.setText(parser.sep)
        annots = f'{self.SEP}'.join(parser.annot_names)
        self._annot_edit.setText(annots)
        self._dirty = False

    def draw_object(self, parser: FileNameSepParser):
        self._parser = parser
        self._sep_edit.setText(parser.sep)
        annots = f'{self.SEP}'.join(parser.annot_names)
        self._annot_edit.setText(annots)
        self._dirty = False

    def clear(self):
        self._sep_edit.setText('')
        self._annot_edit.setText('')
        self._dirty = False

    def is_dirty(self) -> bool:
        return self._dirty

    def _get_annots(self):
        annots_text = self._annot_edit.toPlainText()
        annots = annots_text.split(self.SEP)
        return annots

    @QtCore.Slot()
    def _on_sep_changed(self, new_sep: str):
        self._parser.sep = new_sep

    @QtCore.Slot()
    def _on_annots_changed(self):
        annots_names = self._get_annots()
        self._parser.annot_names = annots_names


if __name__ == '__main__':
    import sys
    from PySide2.QtWidgets import QApplication
    from PySide2.QtWidgets import QWidget, QVBoxLayout

    app = QApplication(sys.argv)

    window = QWidget()
    window.setMinimumSize(100, 100)
    layout = QVBoxLayout()
    window.setLayout(layout)

    panel = FileNameParserWidget()
    layout.addWidget(panel)

    window.show()

    sys.exit(app.exec_())
